@@ -5,7 +5,7 @@ Earley’s dynamic lexer
 Demonstrates the power of Earley’s dynamic lexer on a toy configuration language
 
 Using a lexer for configuration files is tricky, because values don't
-have to be surrounded by delimiters. Using a standard lexer for this just won't work.
+have to be surrounded by delimiters. Using a basic lexer for this just won't work.
 
 In this example we use a dynamic lexer and let the Earley parser resolve the ambiguity.
 
@@ -20,9 +20,10 @@ parser = Lark(r"""
         start: _NL? section+
         section: "[" NAME "]" _NL item+
         item: NAME "=" VALUE? _NL
+
+        NAME: /\w/+
         VALUE: /./+
 
-        %import common.CNAME -> NAME
         %import common.NEWLINE -> _NL
         %import common.WS_INLINE
         %ignore WS_INLINE
